@@ -24,13 +24,12 @@ import (
 	"strings"
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	api_errors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/watch"
 
 	"knative.dev/client/pkg/kn/commands"
-	servinglib "knative.dev/client/pkg/serving"
 	"knative.dev/client/pkg/util"
 	"knative.dev/client/pkg/wait"
 	network "knative.dev/networking/pkg"
@@ -138,6 +137,9 @@ func TestServiceCreateImage(t *testing.T) {
 		!strings.Contains(output, commands.FakeNamespace) {
 		t.Fatalf("wrong stdout message: %v", output)
 	}
+
+	// No revision name set by default
+	assert.Assert(t, template.Name == "")
 }
 
 func TestServiceCreateWithMultipleImages(t *testing.T) {
@@ -225,7 +227,7 @@ func TestServiceCreateEnv(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	actualEnvVars, err := servinglib.EnvToMap(template.Spec.Containers[0].Env)
+	actualEnvVars, err := util.EnvToMap(template.Spec.Containers[0].Env)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -241,34 +243,6 @@ func TestServiceCreateEnv(t *testing.T) {
 	}
 }
 
-func TestServiceCreateWithDeprecatedRequests(t *testing.T) {
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
-		"--requests-cpu", "250m", "--requests-memory", "64Mi",
-		"--no-wait"}, false)
-
-	if err != nil {
-		t.Fatal(err)
-	} else if !action.Matches("create", "services") {
-		t.Fatalf("Bad action %v", action)
-	}
-
-	expectedRequestsVars := corev1.ResourceList{
-		corev1.ResourceCPU:    parseQuantity(t, "250m"),
-		corev1.ResourceMemory: parseQuantity(t, "64Mi"),
-	}
-
-	template := &created.Spec.Template
-
-	if err != nil {
-		t.Fatal(err)
-	} else if !reflect.DeepEqual(
-		template.Spec.Containers[0].Resources.Requests,
-		expectedRequestsVars) {
-		t.Fatalf("wrong requests vars %v", template.Spec.Containers[0].Resources.Requests)
-	}
-}
-
 func TestServiceCreateWithRequests(t *testing.T) {
 	action, created, _, err := fakeServiceCreate([]string{
 		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
@@ -297,34 +271,6 @@ func TestServiceCreateWithRequests(t *testing.T) {
 	}
 }
 
-func TestServiceCreateWithDeprecatedLimits(t *testing.T) {
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
-		"--limits-cpu", "1000m", "--limits-memory", "1024Mi",
-		"--no-wait"}, false)
-
-	if err != nil {
-		t.Fatal(err)
-	} else if !action.Matches("create", "services") {
-		t.Fatalf("Bad action %v", action)
-	}
-
-	expectedLimitsVars := corev1.ResourceList{
-		corev1.ResourceCPU:    parseQuantity(t, "1000m"),
-		corev1.ResourceMemory: parseQuantity(t, "1024Mi"),
-	}
-
-	template := &created.Spec.Template
-
-	if err != nil {
-		t.Fatal(err)
-	} else if !reflect.DeepEqual(
-		template.Spec.Containers[0].Resources.Limits,
-		expectedLimitsVars) {
-		t.Fatalf("wrong limits vars %v", template.Spec.Containers[0].Resources.Limits)
-	}
-}
-
 func TestServiceCreateWithLimits(t *testing.T) {
 	action, created, _, err := fakeServiceCreate([]string{
 		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
@@ -353,45 +299,6 @@ func TestServiceCreateWithLimits(t *testing.T) {
 	}
 }
 
-func TestServiceCreateDeprecatedRequestsLimitsCPU(t *testing.T) {
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
-		"--requests-cpu", "250m", "--limits-cpu", "1000m",
-		"--no-wait"}, false)
-
-	if err != nil {
-		t.Fatal(err)
-	} else if !action.Matches("create", "services") {
-		t.Fatalf("Bad action %v", action)
-	}
-
-	expectedRequestsVars := corev1.ResourceList{
-		corev1.ResourceCPU: parseQuantity(t, "250m"),
-	}
-
-	expectedLimitsVars := corev1.ResourceList{
-		corev1.ResourceCPU: parseQuantity(t, "1000m"),
-	}
-
-	template := &created.Spec.Template
-
-	if err != nil {
-		t.Fatal(err)
-	} else {
-		if !reflect.DeepEqual(
-			template.Spec.Containers[0].Resources.Requests,
-			expectedRequestsVars) {
-			t.Fatalf("wrong requests vars %v", template.Spec.Containers[0].Resources.Requests)
-		}
-
-		if !reflect.DeepEqual(
-			template.Spec.Containers[0].Resources.Limits,
-			expectedLimitsVars) {
-			t.Fatalf("wrong limits vars %v", template.Spec.Containers[0].Resources.Limits)
-		}
-	}
-}
-
 func TestServiceCreateRequestsLimitsCPU(t *testing.T) {
 	action, created, _, err := fakeServiceCreate([]string{
 		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
@@ -431,46 +338,6 @@ func TestServiceCreateRequestsLimitsCPU(t *testing.T) {
 	}
 }
 
-func TestServiceCreateDeprecatedRequestsLimitsMemory(t *testing.T) {
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "foo",
-		"--image", "gcr.io/foo/bar:baz",
-		"--requests-memory", "64Mi",
-		"--limits-memory", "1024Mi", "--no-wait"}, false)
-
-	if err != nil {
-		t.Fatal(err)
-	} else if !action.Matches("create", "services") {
-		t.Fatalf("Bad action %v", action)
-	}
-
-	expectedRequestsVars := corev1.ResourceList{
-		corev1.ResourceMemory: parseQuantity(t, "64Mi"),
-	}
-
-	expectedLimitsVars := corev1.ResourceList{
-		corev1.ResourceMemory: parseQuantity(t, "1024Mi"),
-	}
-
-	template := &created.Spec.Template
-
-	if err != nil {
-		t.Fatal(err)
-	} else {
-		if !reflect.DeepEqual(
-			template.Spec.Containers[0].Resources.Requests,
-			expectedRequestsVars) {
-			t.Fatalf("wrong requests vars %v", template.Spec.Containers[0].Resources.Requests)
-		}
-
-		if !reflect.DeepEqual(
-			template.Spec.Containers[0].Resources.Limits,
-			expectedLimitsVars) {
-			t.Fatalf("wrong limits vars %v", template.Spec.Containers[0].Resources.Limits)
-		}
-	}
-}
-
 func TestServiceCreateRequestsLimitsMemory(t *testing.T) {
 	action, created, _, err := fakeServiceCreate([]string{
 		"service", "create", "foo",
@@ -618,11 +485,150 @@ func TestServiceCreateScaleWithMinScaleSet(t *testing.T) {
 
 }
 
+func TestServiceCreateScaleRange(t *testing.T) {
+	action, created, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "1..5", "--no-wait"}, false)
+
+	if err != nil {
+		t.Fatal(err)
+	}
+	if !action.Matches("create", "services") {
+		t.Fatal("Bad action ", action)
+	}
+
+	template := &created.Spec.Template
+
+	actualAnnos := template.Annotations
+	expectedAnnos := []string{
+		"autoscaling.knative.dev/minScale", "1",
+		"autoscaling.knative.dev/maxScale", "5",
+	}
+
+	for i := 0; i < len(expectedAnnos); i += 2 {
+		anno := expectedAnnos[i]
+		if actualAnnos[anno] != expectedAnnos[i+1] {
+			t.Fatalf("Unexpected annotation value for %s : %s (actual) != %s (expected)",
+				anno, actualAnnos[anno], expectedAnnos[i+1])
+		}
+	}
+}
+
+func TestServiceCreateScaleRangeOnlyMin(t *testing.T) {
+	action, created, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "1..", "--no-wait"}, false)
+
+	if err != nil {
+		t.Fatal(err)
+	}
+	if !action.Matches("create", "services") {
+		t.Fatal("Bad action ", action)
+	}
+
+	template := &created.Spec.Template
+
+	actualAnnos := template.Annotations
+	expectedAnnos := []string{
+		"autoscaling.knative.dev/minScale", "1",
+	}
+
+	for i := 0; i < len(expectedAnnos); i += 2 {
+		anno := expectedAnnos[i]
+		if actualAnnos[anno] != expectedAnnos[i+1] {
+			t.Fatalf("Unexpected annotation value for %s : %s (actual) != %s (expected)",
+				anno, actualAnnos[anno], expectedAnnos[i+1])
+		}
+	}
+}
+
+func TestServiceCreateScaleRangeOnlyMinNegative(t *testing.T) {
+	_, _, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "-1..", "--no-wait"}, true)
+	if err == nil {
+		t.Fatal(err)
+	}
+	expectedErrMsg := "expected 0 <= -1 <= 2147483647: autoscaling.knative.dev/minScale"
+	if !strings.Contains(err.Error(), expectedErrMsg) {
+		t.Errorf("Invalid error output, expected: %s, got : '%s'", expectedErrMsg, err)
+	}
+}
+
+func TestServiceCreateScaleRangeOnlyMax(t *testing.T) {
+	action, created, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "..5", "--no-wait"}, false)
+
+	if err != nil {
+		t.Fatal(err)
+	}
+	if !action.Matches("create", "services") {
+		t.Fatal("Bad action ", action)
+	}
+
+	template := &created.Spec.Template
+
+	actualAnnos := template.Annotations
+	expectedAnnos := []string{
+		"autoscaling.knative.dev/maxScale", "5",
+	}
+
+	for i := 0; i < len(expectedAnnos); i += 2 {
+		anno := expectedAnnos[i]
+		if actualAnnos[anno] != expectedAnnos[i+1] {
+			t.Fatalf("Unexpected annotation value for %s : %s (actual) != %s (expected)",
+				anno, actualAnnos[anno], expectedAnnos[i+1])
+		}
+	}
+}
+
+func TestServiceCreateScaleRangeOnlyMaxNegative(t *testing.T) {
+	_, _, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "..-5", "--no-wait"}, true)
+	if err == nil {
+		t.Fatal(err)
+	}
+	expectedErrMsg := "expected 0 <= -5 <= 2147483647: autoscaling.knative.dev/maxScale"
+	if !strings.Contains(err.Error(), expectedErrMsg) {
+		t.Errorf("Invalid error output, expected: %s, got : '%s'", expectedErrMsg, err)
+	}
+}
+
+func TestServiceCreateScaleRangeOnlyMinWrongSeparator(t *testing.T) {
+	_, _, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "1--", "--no-wait"}, true)
+	if err == nil {
+		t.Fatal(err)
+	}
+	expectedErrMsg := "Scale must be of the format x..y or x"
+	if !strings.Contains(err.Error(), expectedErrMsg) {
+		t.Errorf("Invalid error output, expected: %s, got : '%s'", expectedErrMsg, err)
+	}
+
+}
+
+func TestServiceCreateScaleRangeOnlyMaxWrongSeparator(t *testing.T) {
+	_, _, _, err := fakeServiceCreate([]string{
+		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
+		"--scale", "--1", "--no-wait"}, true)
+	if err == nil {
+		t.Fatal(err)
+	}
+	expectedErrMsg := "Scale must be of the format x..y or x"
+	if !strings.Contains(err.Error(), expectedErrMsg) {
+		t.Errorf("Invalid error output, expected: %s, got : '%s'", expectedErrMsg, err)
+	}
+
+}
+
 func TestServiceCreateRequestsLimitsCPUMemory(t *testing.T) {
 	action, created, _, err := fakeServiceCreate([]string{
 		"service", "create", "foo", "--image", "gcr.io/foo/bar:baz",
-		"--requests-cpu", "250m", "--limits-cpu", "1000m",
-		"--requests-memory", "64Mi", "--limits-memory", "1024Mi",
+		"--request", "cpu=250m", "--limit", "cpu=1000m",
+		"--request", "memory=64Mi", "--limit", "memory=1024Mi",
 		"--no-wait"}, false)
 
 	if err != nil {
@@ -726,7 +732,7 @@ func TestServiceCreateEnvForce(t *testing.T) {
 	if err != nil {
 		t.Fatal(err)
 	}
-	actualEnvVars, err := servinglib.EnvToMap(template.Spec.Containers[0].Env)
+	actualEnvVars, err := util.EnvToMap(template.Spec.Containers[0].Env)
 	if err != nil {
 		t.Fatal(err)
 	} else if template.Spec.Containers[0].Image != "gcr.io/foo/bar:v2" {
@@ -823,93 +829,61 @@ var serviceJSON = `
   }
 }`
 
-func TestServiceCreateFromYAML(t *testing.T) {
-	tempDir, err := ioutil.TempDir("", "kn-file")
-	defer os.RemoveAll(tempDir)
-	assert.NilError(t, err)
+func TestServiceCreateFromFile(t *testing.T) {
+	testWithServiceFiles(t, func(t *testing.T, tempFile string) {
+		for _, testArgs := range [][]string{
+			{
+				"service", "create", "foo", "--filename", tempFile}, {
+				"service", "create", "--filename", tempFile},
+		} {
+			action, created, _, err := fakeServiceCreate(testArgs, false)
+			assert.NilError(t, err)
+			assert.Assert(t, action.Matches("create", "services"))
 
-	tempFile := filepath.Join(tempDir, "service.yaml")
-	err = ioutil.WriteFile(tempFile, []byte(serviceYAML), os.FileMode(0666))
-	assert.NilError(t, err)
-
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "foo", "--filename", tempFile}, false)
-	assert.NilError(t, err)
-	assert.Assert(t, action.Matches("create", "services"))
-
-	assert.Equal(t, created.Name, "foo")
-	assert.Equal(t, created.Spec.Template.Spec.GetContainer().Image, "gcr.io/foo/bar:baz")
+			assert.Equal(t, created.Name, "foo")
+			assert.Equal(t, created.Spec.Template.Spec.GetContainer().Image, "gcr.io/foo/bar:baz")
+		}
+	})
 }
 
-func TestServiceCreateFromJSON(t *testing.T) {
+func testWithServiceFiles(t *testing.T, testFunction func(t *testing.T, file string)) {
 	tempDir, err := ioutil.TempDir("", "kn-file")
 	defer os.RemoveAll(tempDir)
 	assert.NilError(t, err)
 
-	tempFile := filepath.Join(tempDir, "service.json")
-	err = ioutil.WriteFile(tempFile, []byte(serviceJSON), os.FileMode(0666))
-	assert.NilError(t, err)
-
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "foo", "--filename", tempFile}, false)
-	assert.NilError(t, err)
-	assert.Assert(t, action.Matches("create", "services"))
-
-	assert.Equal(t, created.Name, "foo")
-	assert.Equal(t, created.Spec.Template.Spec.GetContainer().Image, "gcr.io/foo/bar:baz")
-}
-
-func TestServiceCreateFromFileWithName(t *testing.T) {
-	tempDir, err := ioutil.TempDir("", "kn-file")
-	defer os.RemoveAll(tempDir)
-	assert.NilError(t, err)
-
-	tempFile := filepath.Join(tempDir, "service.yaml")
-	err = ioutil.WriteFile(tempFile, []byte(serviceYAML), os.FileMode(0666))
-	assert.NilError(t, err)
-
-	t.Log("no NAME param provided")
-	action, created, _, err := fakeServiceCreate([]string{
-		"service", "create", "--filename", tempFile}, false)
-	assert.NilError(t, err)
-	assert.Assert(t, action.Matches("create", "services"))
-
-	assert.Equal(t, created.Name, "foo")
-	assert.Equal(t, created.Spec.Template.Spec.GetContainer().Image, "gcr.io/foo/bar:baz")
-
-	t.Log("no service.Name provided in file")
-	err = ioutil.WriteFile(tempFile, []byte(strings.ReplaceAll(serviceYAML, "name: foo", "")), os.FileMode(0666))
-	assert.NilError(t, err)
-	action, created, _, err = fakeServiceCreate([]string{
-		"service", "create", "cli-foo", "--filename", tempFile}, false)
-	assert.NilError(t, err)
-	assert.Assert(t, action.Matches("create", "services"))
-
-	assert.Equal(t, created.Name, "cli-foo")
-	assert.Equal(t, created.Spec.Template.Spec.GetContainer().Image, "gcr.io/foo/bar:baz")
+	for _, d := range []struct {
+		filename string
+		content  string
+	}{
+		{"service.yaml",
+			serviceYAML,
+		},
+		{"service.json",
+			serviceJSON,
+		},
+	} {
+		tempFile := filepath.Join(tempDir, d.filename)
+		err = ioutil.WriteFile(tempFile, []byte(d.content), os.FileMode(0666))
+		assert.NilError(t, err)
+		testFunction(t, tempFile)
+	}
 }
 
 func TestServiceCreateFileNameMismatch(t *testing.T) {
-	tempDir, err := ioutil.TempDir("", "kn-file")
-	assert.NilError(t, err)
+	testWithServiceFiles(t, func(t *testing.T, tempFile string) {
+		_, _, _, err := fakeServiceCreate([]string{
+			"service", "create", "anotherFoo", "--filename", tempFile}, false)
+		assert.Assert(t, err != nil)
+		assert.Assert(t, util.ContainsAllIgnoreCase(err.Error(), "provided", "'anotherFoo'", "name", "match", "from", "file", "'foo'"))
 
-	tempFile := filepath.Join(tempDir, "service.json")
-	err = ioutil.WriteFile(tempFile, []byte(serviceJSON), os.FileMode(0666))
-	assert.NilError(t, err)
+		err = ioutil.WriteFile(tempFile, []byte(strings.ReplaceAll(serviceYAML, "name: foo", "")), os.FileMode(0666))
+		assert.NilError(t, err)
+		_, _, _, err = fakeServiceCreate([]string{
+			"service", "create", "--filename", tempFile}, false)
+		assert.Assert(t, err != nil)
+		assert.Assert(t, util.ContainsAllIgnoreCase(err.Error(), "no", "service", "name", "provided", "parameter", "file"))
 
-	t.Log("NAME param nad service.Name differ")
-	_, _, _, err = fakeServiceCreate([]string{
-		"service", "create", "anotherFoo", "--filename", tempFile}, false)
-	assert.Assert(t, err != nil)
-	assert.Assert(t, util.ContainsAllIgnoreCase(err.Error(), "provided", "'anotherFoo'", "name", "match", "from", "file", "'foo'"))
-
-	t.Log("no NAME param & no service.Name provided in file")
-	err = ioutil.WriteFile(tempFile, []byte(strings.ReplaceAll(serviceYAML, "name: foo", "")), os.FileMode(0666))
-	assert.NilError(t, err)
-	_, _, _, err = fakeServiceCreate([]string{
-		"service", "create", "--filename", tempFile}, false)
-	assert.Assert(t, err != nil)
-	assert.Assert(t, util.ContainsAllIgnoreCase(err.Error(), "no", "service", "name", "provided", "parameter", "file"))
+	})
 }
 
 func TestServiceCreateFileError(t *testing.T) {
@@ -993,7 +967,7 @@ func TestServiceCreateFromYAMLWithOverride(t *testing.T) {
 	assert.Assert(t, action.Matches("create", "services"))
 	assert.Equal(t, created.Name, "foo")
 
-	actualEnvVar, err := servinglib.EnvToMap(created.Spec.Template.Spec.GetContainer().Env)
+	actualEnvVar, err := util.EnvToMap(created.Spec.Template.Spec.GetContainer().Env)
 	assert.NilError(t, err)
 	assert.DeepEqual(t, actualEnvVar, expectedEnvVars)
 
@@ -1007,7 +981,7 @@ func TestServiceCreateFromYAMLWithOverride(t *testing.T) {
 	assert.Assert(t, action.Matches("create", "services"))
 	assert.Equal(t, created.Name, "foo")
 
-	actualEnvVar, err = servinglib.EnvToMap(created.Spec.Template.Spec.GetContainer().Env)
+	actualEnvVar, err = util.EnvToMap(created.Spec.Template.Spec.GetContainer().Env)
 	assert.NilError(t, err)
 	assert.DeepEqual(t, actualEnvVar, expectedEnvVars)
 
@@ -1020,7 +994,7 @@ func TestServiceCreateFromYAMLWithOverride(t *testing.T) {
 	assert.Assert(t, action.Matches("create", "services"))
 	assert.Equal(t, created.Name, "foo")
 
-	actualEnvVar, err = servinglib.EnvToMap(created.Spec.Template.Spec.GetContainer().Env)
+	actualEnvVar, err = util.EnvToMap(created.Spec.Template.Spec.GetContainer().Env)
 	assert.NilError(t, err)
 	assert.DeepEqual(t, actualEnvVar, expectedEnvVars)
 
